
/* DemoSwingApplet: Demonstrating GUI + Event Handling in Swing */

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

interface IGeometryConstants {                                    // (1)
    int SQUARE  = 0;
    int CIRCLE  = 1;
    int ELLIPSE = 2;
    String[] shapeNames = {"Square", "Circle", "Ellipse"};

    int SMALL  = 0;
    int MEDIUM = 1;
    int LARGE  = 2;
    String[] sizeNames = {"Small", "Medium", "Large"};
}

public class DemoSwingApplet extends JApplet implements IGeometryConstants {

    // Panel for shape                                               (2)
    JPanel       shapePanel;
    ButtonGroup  shapeGroup;
    JRadioButton squareRB;
    JRadioButton circleRB;
    JRadioButton ellipseRB;

    // Panel for x, y coordinates
    JPanel     xyPanel;
    JLabel     xLabel;
    JTextField xInput;
    JLabel     yLabel;
    JTextField yInput;

    // Panel for size and fill
    JPanel    sizePanel;
    JLabel    sizeLabel;
    JComboBox sizeChoices;
    JCheckBox fillCB;

    // Box for shape, coordinates, size and fill
    Box leftBox;

    // Panel for draw button, drawing region and message display.
    JPanel      rightPanel;
    JButton     drawButton;
    JScrollPane scrollPane;
    DrawRegion  drawRegion;
    JTextField  messageDisplay;

    // Top container
    Container topContainer;

    public void init() {                                          // (3)
        makeShapePanel();
        makeXYPanel();
        makeSizePanel();
        makeLeftBox();
        makeRightPanel();

        addListeners();                                           // (4)

        topContainer = getContentPane();                          // (5)
        topContainer.add(leftBox,    BorderLayout.WEST);
        topContainer.add(rightPanel, BorderLayout.CENTER);

        try {                                                     // (6)
            UIManager.setLookAndFeel(
                UIManager.getSystemLookAndFeelClassName()
//              "com.sun.java.swing.plaf.motif.MotifLookAndFeel"
//              "com.sun.java.swing.plaf.windows.WindowsLookAndFeel"
//              "javax.swing.plaf.mac.MacLookAndFeel"
            );
        } catch (Exception e) { messageDisplay.setText(e.toString()); }
        SwingUtilities.updateComponentTreeUI(this);               // (7)
    }

    // Panel for shape                                               (8)
    void makeShapePanel() {
        squareRB  = new JRadioButton(shapeNames[SQUARE],  true);
        circleRB  = new JRadioButton(shapeNames[CIRCLE],  false);
        ellipseRB = new JRadioButton(shapeNames[ELLIPSE], false);

        shapeGroup = new ButtonGroup();
        shapeGroup.add(squareRB);
        shapeGroup.add(circleRB);
        shapeGroup.add(ellipseRB);

        shapePanel = new JPanel();
        shapePanel.setLayout(new FlowLayout());
        shapePanel.add(squareRB);
        shapePanel.add(circleRB);
        shapePanel.add(ellipseRB);
        shapePanel.setBorder(new TitledBorder("Shapes"));
    }

    // Panel for x,y coordinates                                     (9)
    void makeXYPanel() {
        xyPanel = new JPanel();

        xLabel  = new JLabel("X Coordinate:");
        yLabel  = new JLabel("Y Coordinate:");

        xInput  = new JTextField(5);
        yInput  = new JTextField(5);

        xyPanel.setLayout(new GridLayout(2,2));
        xyPanel.add(xLabel);
        xyPanel.add(xInput);
        xyPanel.add(yLabel);
        xyPanel.add(yInput);
    }

    // Panel for size and fill                                       (10)
    void makeSizePanel() {

        sizePanel   = new JPanel();

        sizeLabel   = new JLabel("Size:");

        sizeChoices = new JComboBox(sizeNames);
        sizeChoices.setSelectedIndex(0);

        fillCB = new JCheckBox("Fill", false);

        sizePanel.setLayout(new FlowLayout());
        sizePanel.add(sizeLabel);
        sizePanel.add(sizeChoices);
        sizePanel.add(fillCB);
    }

    // Box for shape, coordinates, size and fill                     (11)
    void makeLeftBox() {
        leftBox = new Box(BoxLayout.Y_AXIS);

        leftBox.add(shapePanel);
        leftBox.add(Box.createGlue());
        leftBox.add(xyPanel);
        leftBox.add(Box.createGlue());
        leftBox.add(sizePanel);
    }

    // Panel for message display, draw button, and drawing region    (12)
    void makeRightPanel() {
        rightPanel = new JPanel();

        messageDisplay = new JTextField("MESSAGE DISPLAY");
        messageDisplay.setEditable(false);

        drawButton = new JButton("Draw");

        drawRegion = new DrawRegion();
        scrollPane = new JScrollPane(drawRegion);

        rightPanel.setLayout(new BorderLayout());
        rightPanel.add(drawButton, BorderLayout.NORTH);
        rightPanel.add(messageDisplay, BorderLayout.SOUTH);
        rightPanel.add(scrollPane, BorderLayout.CENTER);
    }

    // Add the listeners.
    void addListeners() {

        drawButton.addActionListener(new ActionListener() {       // (13)
            public void actionPerformed(ActionEvent evt) {

                int shape, xCoord, yCoord, width;

                messageDisplay.setText("");
                // Get the shape                                     (14)
                if (squareRB.isSelected())
                    shape = SQUARE;
                else if (circleRB.isSelected())
                    shape = CIRCLE;
                else if (ellipseRB.isSelected())
                    shape = ELLIPSE;
                else {
                    messageDisplay.setText("Unknown shape.");
                    return;
                }

                // Get the coordinates                               (15)
                try {
                    xCoord = Integer.parseInt(xInput.getText());
                    yCoord = Integer.parseInt(yInput.getText());
                } catch (NumberFormatException e) {
                    messageDisplay.setText("Illegal coordinates.");
                    return;
                }

                // Get the size                                      (16)
                switch (sizeChoices.getSelectedIndex()) {
                    case SMALL:  width = 30;  break;
                    case MEDIUM: width = 60;  break;
                    case LARGE:  width = 120; break;
                    default: messageDisplay.setText("Unknown size."); return;
                }

                messageDisplay.setText("Drawing " + shapeNames[shape]);
                drawRegion.doDraw(                                // (17)
                        shape,
                        xCoord, yCoord,
                        fillCB.isSelected(),
                        width
                );
            }
        });
    }
}

class DrawRegion extends JPanel implements IGeometryConstants {   // (18)

    // Values needed for drawing the shape
    private int     shape;
    private int     xCoord;
    private int     yCoord;
    private boolean fillFlag;
    private int     width;

    // Default constructor
    public DrawRegion() {
        setSize(300,300);
        setBackground(Color.white);
    }

    // Overridden to always keep the same size
    public Dimension getPreferredSize() {
        return getSize();
    }

    // Set the values and repaint the drawing region.                (19)
    public void doDraw(int shape,
                       int xCoord, int yCoord,
                       boolean fillFlag, int width) {

        this.shape    = shape;
        this.xCoord   = xCoord;
        this.yCoord   = yCoord;
        this.fillFlag = fillFlag;
        this.width    = width;

        repaint();                                                // (20)
    }

    // Do the drawing of the shape                                   (21)
    public void paintComponent (Graphics g) {
        super.paintComponent(g);
        switch (shape) {
            case SQUARE:
                if (fillFlag) g.fillRect(xCoord, yCoord, width, width);
                else g.drawRect(xCoord, yCoord, width, width);
                break;
            case CIRCLE:
                if (fillFlag) g.fillOval(xCoord, yCoord, width, width);
                else g.drawOval(xCoord, yCoord, width, width);
                break;
            case ELLIPSE:
                if (fillFlag) g.fillOval(xCoord, yCoord, width, width/2);
                else g.drawOval(xCoord, yCoord, width, width/2);
                break;
        }
    }
}
