
// Filename: DemoSwingApplet.java

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.*;

/**
 * An applet demonstrating how GUIs are built and how event handling
 * works in Swing. This is a Swing version of an earlier AWT applet.
 *
 * <p>The user can specify the shape, size, fill and coordinates for a
 * drawing.  Clicking the draw button renders the shape according to
 * the specificatons indicated by the user.
 *
 * <p>The drawing region is implemented by the class
 * <code>DrawRegion</code>.
 *
 * @see DrawRegion
 */
public class DemoSwingApplet extends JApplet implements IGeometryConstants {

    /** Panel where the user can select the shape. */
    JPanel       shapePanel;
    /**
     * The group of radio buttons which allow the user to select the shape.
     */
    ButtonGroup  shapeGroup;
    /** For selecting the square shape. */
    JRadioButton squareRB;
    /** For selecting the circle shape. */
    JRadioButton circleRB;
    /** For selecting the ellipse shape. */
    JRadioButton ellipseRB;

    /** Panel where the user can specify x, y coordinates. */
    JPanel     xyPanel;
    JLabel     xLabel;
    /** For editing the x coordinate. */
    JTextField xInput;
    JLabel     yLabel;
    /** For editing the y coordinate. */
    JTextField yInput;

    /** Panel where the user can select size and fill. */
    JPanel    sizePanel;
    JLabel    sizeLabel;
    /** List of size choices the user can select. */
    JComboBox sizeChoices;
    JCheckBox fillCB;

    /**
     * Box for laying out the shape, coordinates, size and fill panels.
     */
    Box leftBox;

    /** Panel for the draw button, drawing region and message display. */
    JPanel      rightPanel;
    /** The button that will activate drawing of the shape. */
    JButton     drawButton;
    /**
     * A scroll pane allowing the draw region to be scrolled by the user.
     */
    JScrollPane scrollPane;
    /** The drawing region where the shape will be drawn. */
    DrawRegion  drawRegion;
    /** A text field used to report messages. */
    JTextField  messageDisplay;

    /** The top container. */
    Container topContainer;

    /**
     * Initialize the applet by creating all the GUI components.
     */
    public void init() {                                          // (3)
        makeShapePanel();
        makeXYPanel();
        makeSizePanel();
        makeLeftBox();
        makeRightPanel();

        addListeners();                                           // (4)

        topContainer = getContentPane();                          // (5)
        topContainer.add(leftBox,    BorderLayout.WEST);
        topContainer.add(rightPanel, BorderLayout.CENTER);

        try {                                                     // (6)
            UIManager.setLookAndFeel(
                UIManager.getSystemLookAndFeelClassName()
//              "com.sun.java.swing.plaf.motif.MotifLookAndFeel"
//              "com.sun.java.swing.plaf.windows.WindowsLookAndFeel"
//              "javax.swing.plaf.mac.MacLookAndFeel"
            );
        } catch (Exception e) { messageDisplay.setText(e.toString()); }
        SwingUtilities.updateComponentTreeUI(this);               // (7)
    }

    /**
     * Creates a panel with radio buttons where the user can select
     * the shape type.  Uses a <code>ButtonGroup</code> instance to
     * provide the semantics of radio buttons.  Stores the result in
     * the member variables.  This method is called by the
     * <code>init()</code> method.
     *
     * @see #shapePanel
     * @see #shapeBG
     * @see #squareRB
     * @see #ellipseRB
     * @see #init
     */
    void makeShapePanel() {
        squareRB  = new JRadioButton(shapeNames[SQUARE],  true);
        circleRB  = new JRadioButton(shapeNames[CIRCLE],  false);
        ellipseRB = new JRadioButton(shapeNames[ELLIPSE], false);

        shapeGroup = new ButtonGroup();
        shapeGroup.add(squareRB);
        shapeGroup.add(circleRB);
        shapeGroup.add(ellipseRB);

        shapePanel = new JPanel();
        shapePanel.setLayout(new FlowLayout());
        shapePanel.add(squareRB);
        shapePanel.add(circleRB);
        shapePanel.add(ellipseRB);
        shapePanel.setBorder(new TitledBorder("Shapes"));
    }

    /**
     * Creates a panel with components where the user can specify x, y
     * coordinates.  Stores the result in the member variables.  This
     * method is called by the <code>init()</code> method.
     *
     * @see #xyPanel
     * @see #xInput
     * @see #yInput
     * @see #init
     */
    void makeXYPanel() {
        xyPanel = new JPanel();

        xLabel  = new JLabel("X Coordinate:");
        yLabel  = new JLabel("Y Coordinate:");

        xInput  = new JTextField(5);
        yInput  = new JTextField(5);

        xyPanel.setLayout(new GridLayout(2,2));
        xyPanel.add(xLabel);
        xyPanel.add(xInput);
        xyPanel.add(yLabel);
        xyPanel.add(yInput);
    }

    /**
     * Creates a panel with components where the user can select size
     * and fill.  The size can be selected from a list of choices.
     * Stores the result in member variables.  This method is called
     * by the <code>init()</code> method.
     *
     * @see #sizePanel
     * @see #sizeChoices
     * @see #fillCB
     * @see #init
     */
    void makeSizePanel() {

        sizePanel   = new JPanel();

        sizeLabel   = new JLabel("Size:");

        sizeChoices = new JComboBox(sizeNames);
        sizeChoices.setSelectedIndex(0);

        fillCB = new JCheckBox("Fill", false);

        sizePanel.setLayout(new FlowLayout());
        sizePanel.add(sizeLabel);
        sizePanel.add(sizeChoices);
        sizePanel.add(fillCB);
    }

   /**
    * Lays out the shape, coordinates, size and fill panels vertically
    * in a box.  Stores the result in a member variable.  This method
    * is called by the <code>init()</code> method.
    *
    * @see #boxA
    * @see #init
    */
    void makeLeftBox() {
        leftBox = new Box(BoxLayout.Y_AXIS);

        leftBox.add(shapePanel);
        leftBox.add(Box.createGlue());
        leftBox.add(xyPanel);
        leftBox.add(Box.createGlue());
        leftBox.add(sizePanel);
    }

    /**
     * Lays out the draw button, drawing region and message display
     * components in a panel.  Stores the result in a member variable.
     * This method is called by the <code>init()</code> method.
     *
     * @see #rightPanel
     * @see #init
     */
    void makeRightPanel() {
        rightPanel = new JPanel();

        messageDisplay = new JTextField("MESSAGE DISPLAY");
        messageDisplay.setEditable(false);

        drawButton = new JButton("Draw");

        drawRegion = new DrawRegion();
        scrollPane = new JScrollPane(drawRegion);

        rightPanel.setLayout(new BorderLayout());
        rightPanel.add(drawButton, BorderLayout.NORTH);
        rightPanel.add(messageDisplay, BorderLayout.SOUTH);
        rightPanel.add(scrollPane, BorderLayout.CENTER);
    }

    /**
     * Adds a listener to the draw button.  The only event handling
     * that takes place is when the user clicks the draw button. An
     * action listener is added to the button. It successively
     * collects all the information required to do the drawing: the
     * kind of shape, the coordinates, the size. Then it finally calls
     * the <code>doDraw()</code> method of the drawing region.  This
     * method is called by the <code>init()</code> method.
     *
     * @see DrawRegion#doDraw
     * @see #init
     */
    void addListeners() {

        drawButton.addActionListener(new ActionListener() {       // (13)
            public void actionPerformed(ActionEvent evt) {

                int shape, xCoord, yCoord, width;

                messageDisplay.setText("");
                // Get the shape                                     (14)
                if (squareRB.isSelected())
                    shape = SQUARE;
                else if (circleRB.isSelected())
                    shape = CIRCLE;
                else if (ellipseRB.isSelected())
                    shape = ELLIPSE;
                else {
                    messageDisplay.setText("Unknown shape.");
                    return;
                }

                // Get the coordinates                               (15)
                try {
                    xCoord = Integer.parseInt(xInput.getText());
                    yCoord = Integer.parseInt(yInput.getText());
                } catch (NumberFormatException e) {
                    messageDisplay.setText("Illegal coordinates.");
                    return;
                }

                // Get the size                                      (16)
                switch (sizeChoices.getSelectedIndex()) {
                    case SMALL:  width = 30;  break;
                    case MEDIUM: width = 60;  break;
                    case LARGE:  width = 120; break;
                    default: messageDisplay.setText("Unknown size."); return;
                }

                messageDisplay.setText("Drawing " + shapeNames[shape]);
                drawRegion.doDraw(                                // (17)
                        shape,
                        xCoord, yCoord,
                        fillCB.isSelected(),
                        width
                );
            }
        });
    }
}
